package Dagger;
#
# $Id: Dagger.pm,v 1.1 2001/10/01 21:34:50 voeckler Exp voeckler $
#
require 5.6.0;
use strict;
use vars qw(@ISA @EXPORT @EXPORT_OK $VERSION);
use Exporter;
@ISA = qw(Exporter);
@EXPORT = qw();
@EXPORT_OK = qw($VERSION);

$VERSION = '0.01';

# ---------------------------------------------------------------
use Carp;

#
# ctor
#
sub new {
    # purpose: create a new Count object for creating DAGs
    # paramtr: -
    my $proto = shift;
    my $class = ref($proto) || $proto || 'Dagger';

    # dag stuff
    my $log = shift;
    
    # create instance
    my $self = bless { 
	m_log => $log,		# log filename
	m_count => 0,		# count for dag node names
	m_visited => {},	# dfs visited nodes set, also node jobfilename
	m_parents => {},	# input file parent nodes
	m_cache => {}		# pfn translation lookup cache
    }, $class;

    # return
    $self;
}

#
# accessors
#
sub log {
    my $self = shift;
    my $oldv = $self->{m_log};
    $self->{m_log} = shift if ( @_ );
    $oldv;
}
    
sub count {
    my $self = shift;
    my $oldv = $self->{m_count};
    $self->{m_count} = shift if ( @_ );
    $oldv;
}

#
# generic member functions
#
sub name {
    # purpose: obtain current value of counter
    # returns: value as one of A, B, ... Z, BA, BB, .. BAA, BAB, ...
    #          mind that A resembles 0, and thus AAA is still 0
    use integer;
    my $self = shift;
    my $x = shift || $self->{m_count};
    my $A = ord('A');
    my @r;
    do {
	unshift @r, chr($A + ($x % 26));
	$x /= 26;
    } while ( $x );
    join q{},@r;
}

sub check {
    # purpose: return cached value for logical to physical filename lookup
    # paramtr: $lfn (IN): lfn to check cache for
    # returns: undef if not found, chosen pfn otherwise
    my $self = shift;
    my $lfn = shift;
    return $self->{m_cache}->{$lfn};
}

sub store {
    # purpose: put a value into the cache
    # paramtr: $lfn (IN): logical filename
    #          $pfn (IN): physical filename to use
    # returns: physical filename
    # warning: dies, if cached hits does not match
    my $self = shift;
    my $lfn = shift;
    my $pfn = shift;
    if ( exists $self->{m_cache}->{$lfn} ) {
	carp( $self->{m_cache}->{$lfn}, " does not match $pfn" )
	    unless $self->{m_cache}->{$lfn} eq $pfn;
    } else {
	$self->{m_cache}->{$lfn} = $pfn;
    }
    $pfn;
}

#
# return 'true' to package loader
#
1;
